/************************  Filename: platform.h  *****************************/
/* ========================================================================= */
/*                                                                           */
/* 0000  000   000  00000 0  000  0   0 0 0000                               */
/* 0   0 0  0 0   0 0     0 0   0 0   0 0 0   0                              */
/* 0   0 0  0 0   0 0     0 0     0   0 0 0   0      Einsteinstrae 6        */
/* 0000  000  0   0 000   0 0     00000 0 0000       91074 Herzogenaurach    */
/* 0     00   0   0 0     0 0     0   0 0 0                                  */
/* 0     0 0  0   0 0     0 0   0 0   0 0 0          Tel: ++49-9132-744-200  */
/* 0     0  0  000  0     0  000  0   0 0 0    GmbH  Fax: ++49-9132-744-204  */
/*                                                                           */
/* ========================================================================= */
/*                                                                           */
/* Description: User defines ( microcontroller, data types, ... ).           */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Technical support:       P. Fredehorst                                    */
/*                          Tel. : ++49-9132/744-214                         */
/*                          Fax. :              -204                         */
/*                          eMail: pfredehorst@profichip.com                 */
/*                          		   support@profichip.com                 */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/* contents:

    - header include hierarchy for system environment

*/
/*****************************************************************************/
/* reinclude protection */


#ifndef PLATFORM_H
#define PLATFORM_H


/*****************************************************************************/
/* header include hierarchy for system environment */

/*---------------------------------------------------------------------------*/
/* COMPILER                                                                  */
/*---------------------------------------------------------------------------*/
#define _KEIL_C51
//#undef _KEIL_C51

/*---------------------------------------------------------------------------*/
/* Hardwareplatform                                                          */
/*---------------------------------------------------------------------------*/
#define EvaBoard_AT89C5132

/*---------------------------------------------------------------------------*/
/* include hierarchy, data types                                             */
/*---------------------------------------------------------------------------*/
#include "..\PLATFORM\AT89C51\regsnd1.h"
#include "..\PLATFORM\AT89C51\extsnd1.h"

//#define AT89C5132_MODE_X2
#undef AT89C5132_MODE_X2

//#define RS232_SERIO
#undef RS232_SERIO

#define LCD
//#undef LCD

// sign the byte order of a word in your device
// possible: LITTLE_ENDIAN   lower address  - high byte
//                           higher address - low byte
//           BIG_ENDIAN      lower address  - low byte
//                           higher address - high byte

#define LITTLE_ENDIAN
#undef  BIG_ENDIAN

#include <intrins.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define TRUE                    1
#define FALSE                   !(TRUE)

#define UBYTE                   unsigned char
#define UWORD                   unsigned int
#define BOOL                    unsigned char
#define ULONG                   unsigned long

#define PTR_ATTR                 xdata
#define VPC3_PTR                 PTR_ATTR *
#define VPC3_ADR                 UWORD
#define VPC3_UNSIGNED8_PTR       UBYTE PTR_ATTR *
#define NULL_PTR 				(void VPC3_PTR)0

#define ROMCONST__ 				code
#define NOP__					_nop_

#include "dp_cfg.h"            				// default configuration
#include "..\dpv0_drv\dpl_list.h"           // double pointered list, makros
#include "..\dpv0_drv\dp_if.h"             	// mpi structure, makros
#include "..\usr_demo\DPV1AFFE\dp_debug.h"

#ifdef LCD
	#define LCD_PTR             PTR_ATTR *

	#include "..\PLATFORM\AT89C51\lcd.h"

	extern void LCDUpdateWord	( UBYTE bAddress, UWORD wValue );
	extern void LCDDelay		( UBYTE bDelay );
#endif//#ifdef LCD

#ifdef RS232_SERIO
	#include "..\PLATFORM\AT89C51\serio.h"
#endif//#ifdef RS232_SERIO


/*---------------------------------------------------------------------------*/
/* hardware settings                                                         */
/*---------------------------------------------------------------------------*/

/*------------------------------------------------------------------------*/
/* VPC3 reset settings                                                    */
/*------------------------------------------------------------------------*/
sbit RES_VPC3 						= 0x90;			// hardware port pin 1.0
#define VPC3_RESET__         		RES_VPC3 = 1
#define VPC3_CLEAR_RESET__			RES_VPC3 = 0

/*------------------------------------------------------------------------*/
/* MPI12x interrupt settings                                              */
/*------------------------------------------------------------------------*/
#define ENABLE_VPC3_INT__			EX0 = 1
#define DISABLE_VPC3_INT__    	    EX0 = 0
#define SET_FALLING_EDGE__   	    IT0 = 1
#define SET_HIGH_PRIORITY__
                                	
/*------------------------------------------------------------------------*/
/* LEDs					                                                  */
/*------------------------------------------------------------------------*/
sbit LED_RED                		= 0x94;			// hardware port pin 1.4
sbit LED_YLW                		= 0x95;			// hardware port pin 1.5
                                	
#define TOGGLE_LED_RED__    		LED_RED ^= 1
#define CLR_LED_RED__       		LED_RED  = 1
#define SET_LED_RED__       		LED_RED  = 0
                                	
#define TOGGLE_LED_YLW__    		LED_YLW ^= 1
#define CLR_LED_YLW__       		LED_YLW  = 1
#define SET_LED_YLW__       		LED_YLW  = 0

/*------------------------------------------------------------------------*/
/* Testpins				                                                  */
/*------------------------------------------------------------------------*/
sbit TEST_PIN_6						= 0x96;			// hardware port pin 1.6
sbit TEST_PIN_7						= 0x97;			// hardware port pin 1.7
                                	
#define TOGGLE_TEST6__      		TEST_PIN_6 ^= 1
#define CLR_TEST6__         		TEST_PIN_6  = 0
#define SET_TEST6__         		TEST_PIN_6  = 1

#define TOGGLE_TEST7__      		TEST_PIN_7 ^= 1
#define CLR_TEST7__         		TEST_PIN_7  = 0
#define SET_TEST7__         		TEST_PIN_7  = 1

/*------------------------------------------------------------------------*/
/* LWL interface macro                                                    */
/*------------------------------------------------------------------------*/
sbit VPC3_ADR10                		= 0xB3;			// hardware port pin 3.3

sbit DIR_AB                 		= 0xB4;			// hardware port pin 3.4
sbit EN_LWL_B               		= 0xB5;			// hardware port pin 3.5

#define DISABLE_LWL_INTERFACE_B__ 	EN_LWL_B = 0
#define ENABLE_LWL_INTERFACE_B__  	EN_LWL_B = 1
#define SET_DIR_AB__              	DIR_AB   = 0
#define SET_DIR_BA__              	DIR_AB   = 1

#define SET_VPC3_ADR10__			VPC3_ADR10 = 1
#define CLR_VPC3_ADR10__			VPC3_ADR10 = 0

#define DISABLE_ALL_INTERRUPT_BIT__	EA = 0
#define ENABLE_ALL_INTERRUPT_BIT__	EA = 1

/*------------------------------------------------------------------------*/
/* XRAM area                                                              */
/*------------------------------------------------------------------------*/
#define VPC3_ASIC_ADDRESS    		((unsigned char *)0x28000L)

#define RTC_ADDRESS					((unsigned char *)0x2B000L)
#define LCD_ADDRESS					((unsigned char *)0x2C000L)
				            		
#define READ_PORT0          		((unsigned char *)0x2D000L)
#define READ_PORT1          		((unsigned char *)0x2E000L)
#define READ_PORT2          		((unsigned char *)0x2F000L)
                            		
#define WRITE_PORT0         		((unsigned char *)0x2D000L)
#define WRITE_PORT1         		((unsigned char *)0x2E000L)
#define WRITE_PORT2					((unsigned char *)0x2F000L)



/*****************************************************************************/
/* reinclude-protection */


#else
    #pragma message "The header PLATFORM.H is included twice or more !"
#endif


/*****************************************************************************/
/*  Copyright (C) profichip GmbH 2005. Confidential.                         */
/*****************************************************************************/