/***********************  Filename: dp_isr.c  ********************************/
/* ========================================================================= */
/*                                                                           */
/* 0000  000   000  00000 0  000  0   0 0 0000                               */
/* 0   0 0  0 0   0 0     0 0   0 0   0 0 0   0                              */
/* 0   0 0  0 0   0 0     0 0     0   0 0 0   0      Einsteinstrae 6        */
/* 0000  000  0   0 000   0 0     00000 0 0000       91074 Herzogenaurach    */
/* 0     00   0   0 0     0 0     0   0 0 0                                  */
/* 0     0 0  0   0 0     0 0   0 0   0 0 0          Tel: ++49-9132-744-200  */
/* 0     0  0  000  0     0  000  0   0 0 0    GmbH  Fax: ++49-9132-744-204  */
/*                                                                           */
/* ========================================================================= */
/*                                                                           */
/* Function:       interrupt service routine for VPC3+                       */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Technical support:       P. Fredehorst                                    */
/*                          Tel. : ++49-9132/744-214                         */
/*                          Fax. :              -204                         */
/*                          eMail: pfredehorst@profichip.com                 */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/* contents:

  - function prototypes
  - data structures
  - internal functions

*/
/*****************************************************************************/
/* include hierarchy */

#include "..\dp_inc\platform.h"
#include "..\dp_inc\dp_inc.h"

/*---------------------------------------------------------------------------*/
/* dp interrupt service routine                                              */
/*---------------------------------------------------------------------------*/
void vpc3_isr( void )
{
	DP_WriteDebugBuffer__( DP_INTERRUPT__, p_Vpc3->isreg.rd.int_reg2, p_Vpc3->isreg.rd.int_reg1 );

    /*-----------------------------------------------------------------------*/
    /* IND_NEW_PRM_DATA                                                      */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_NEW_PRM_DATA == 1 )
        if(VPC3_GET_IND_NEW_PRM_DATA())
        {
			DP_WriteDebugBuffer__( DP_INTERRUPT_PRM__, 0, 0 );
            
            dp_sys.vpc3_con = VPC3_PRM_FINISHED;

            do
            {
                if( user_chk_new_prm_data( VPC3_GET_PRM_BUF_PTR(), VPC3_GET_PRM_LEN() ) == DP_OK )
                {
                    #ifdef DP_MSAC_C1
                        msac_c1_check_ind_new_prm_data();
                    #endif
                    dp_sys.vpc3_con = VPC3_SET_PRM_DATA_OK();
					
					DP_WriteDebugBuffer__( DP_INTERRUPT_PRM__, 0x33, 0 );
                }//if( user_chk_new_prm_data( VPC3_GET_PRM_BUF_PTR(), VPC3_GET_PRM_LEN() ) == DP_OK )
                else
                {
                    dp_sys.vpc3_con = VPC3_SET_PRM_DATA_NOT_OK();
					
					DP_WriteDebugBuffer__( DP_INTERRUPT_PRM__, 0x55, 0 );
                }//else of if( user_chk_new_prm_data( VPC3_GET_PRM_BUF_PTR(), VPC3_GET_PRM_LEN() ) == DP_OK )
            }
            while( dp_sys.vpc3_con == VPC3_PRM_CONFLICT );
        }//if(VPC3_GET_IND_NEW_PRM_DATA())
    #endif//#if( ISR_ENABLE_VPC3_INT_NEW_PRM_DATA == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_NEW_EXT_PRM_DATA                                                  */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_NEW_EXT_PRM_DATA == 1 )
        if(VPC3_GET_IND_NEW_EXT_PRM_DATA())
        {
            dp_sys.vpc3_con = VPC3_PRM_FINISHED;

            do
            {
                if( user_chk_new_ext_prm_data( (VPC3_STRUC_PRM_BLOCK_PTR)VPC3_GET_PRM_BUF_PTR(), VPC3_GET_PRM_LEN() ) == DP_OK )
                {
                    dp_sys.vpc3_con = VPC3_SET_PRM_DATA_OK();
                }//if( user_chk_new_ext_prm_data( VPC3_GET_PRM_BUF_PTR(), VPC3_GET_PRM_LEN() ) == DP_OK )
                else
                {
                    dp_sys.vpc3_con = VPC3_SET_PRM_DATA_NOT_OK();
                }//else of if( user_chk_new_ext_prm_data( VPC3_GET_PRM_BUF_PTR(), VPC3_GET_PRM_LEN() ) == DP_OK )
            }
            while( dp_sys.vpc3_con == VPC3_PRM_CONFLICT );
        }//if(VPC3_GET_IND_NEW_EXT_PRM_DATA())
    #endif//#if( ISR_ENABLE_VPC3_INT_NEW_EXT_PRM_DATA == 1 )

    /*-----------------------------------------------------------------------*/
    /* check config data , application specific!                             */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_NEW_CFG_DATA == 1 )
        if(VPC3_GET_IND_NEW_CFG_DATA())
        {
			DP_WriteDebugBuffer__( DP_INTERRUPT_CFG__, 0, 0 );

            dp_sys.vpc3_con = VPC3_CFG_FINISHED;

            do
            {
                switch( user_chk_new_cfg_data() )
                {
                    case VPC3_CFG_OK:
                    {
                        #ifdef DP_MSAC_C1
                            msac_c1_do_cfg_ok();
                        #endif

                        dp_sys.vpc3_con = VPC3_SET_CFG_DATA_OK();

						DP_WriteDebugBuffer__( DP_INTERRUPT_CFG__, 0x33, 0 );
                        break;
                    }//case VPC3_CFG_OK:

                    case VPC3_CFG_FAULT:
                    {
                        #ifdef DP_MSAC_C1
                            msac_c1_do_cfg_not_ok();
                        #endif
                        dp_sys.vpc3_con = VPC3_SET_CFG_DATA_NOT_OK();

						DP_WriteDebugBuffer__( DP_INTERRUPT_CFG__, 0x44, 0 );
                        break;
                    }//case VPC3_CFG_FAULT:

                    case VPC3_CFG_UPDATE:
                    {
                        // Calculate the length of the input and output using the configuration bytes
                        if( DP_OK != vpc3_calculate_inp_outp_len ( VPC3_GET_CFG_BUF_PTR(), VPC3_GET_CFG_LEN() ) )
                        {
                            #ifdef DP_MSAC_C1
                                msac_c1_do_cfg_not_ok();
                            #endif

                            dp_sys.vpc3_con = VPC3_SET_CFG_DATA_NOT_OK();

						    DP_WriteDebugBuffer__( DP_INTERRUPT_CFG__, 0x55, 0 );
                        }//if( DP_OK != vpc3_calculate_inp_outp_len (cfg_ptr,VPC3_GET_CFG_LEN()) )
                        else
                        {
                            // set IO-Length
                            vpc3_set_io_data_len();

                            #ifdef DP_MSAC_C1
                                msac_c1_do_cfg_ok();
                            #endif

                            dp_sys.vpc3_con = VPC3_SET_CFG_DATA_UPDATE();

							DP_WriteDebugBuffer__( DP_INTERRUPT_CFG__, 0x66, 0 );
                        }//else of if( DP_OK != vpc3_calculate_inp_outp_len (cfg_ptr,VPC3_GET_CFG_LEN()) )
                        break;
                    }//case VPC3_CFG_UPDATE:

                    default:
                    {
                        break;
                    }//default:
                }//switch( user_chk_new_cfg_data() )
            }
            while ( dp_sys.vpc3_con == VPC3_CFG_CONFLICT );
        }//if(VPC3_GET_IND_NEW_CFG_DATA())
    #endif//#if( ISR_ENABLE_VPC3_INT_NEW_CFG_DATA == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_GO_LEAVE_DATA_EX                                                  */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_GO_LEAVE_DATA_EX == 1 )
        if(VPC3_GET_IND_GO_LEAVE_DATA_EX())
        {
            #ifdef DP_MSAC_C1
                msac_c1_leave_dx();
            #endif
            user_isr_go_leave_data_ex();
            VPC3_CON_IND_GO_LEAVE_DATA_EX();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_GO_LEAVE_DATA_EX == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_DX_OUT                                                            */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_DX_OUT == 1 )
        if (VPC3_GET_IND_DX_OUT())
        {
            #ifdef DP_MSAC_C1
                msac_c1_check_ind_dx_out();
            #endif
            user_isr_dx_out();
            VPC3_CON_IND_DX_OUT();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_DX_OUT == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_DIAG_BUF_CHANGED                                                  */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_DIAG_BUF_CHANGED == 1 )
        if(VPC3_GET_IND_DIAG_BUF_CHANGED())
        {
            user_isr_diag_buf_changed();
            VPC3_CON_IND_DIAG_BUF_CHANGED();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_DIAG_BUF_CHANGED == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_WD_DP_TIMEOUT                                                     */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_WD_DP_TIMEOUT == 1 )
        if(VPC3_GET_IND_WD_DP_TIMEOUT() )
        {
            user_isr_new_wd_dp_timeout();
            VPC3_CON_IND_WD_DP_TIMEOUT();    // acknowledge
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_WD_DP_TIMEOUT == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_MAC_RESET                                                         */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_MAC_RESET == 1 )
        if(VPC3_GET_IND_MAC_RESET())
        {
            user_isr_user_mac_reset();
            VPC3_CON_IND_MAC_RESET();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_MAC_RESET == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_BAUDRATE_DETECT                                                   */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_BAUDRATE_DETECT == 1 )
        if(VPC3_GET_IND_BAUDRATE_DETECT())
        {
            #ifdef DP_MSAC_C2
                msac_c2_set_timeout();
            #endif
            user_isr_baudrate_detect();
            VPC3_CON_IND_BAUDRATE_DETECT();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_BAUDRATE_DETECT == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_NEW_GC_COMMAND                                                    */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_NEW_GC_COMMAND == 1 )
        if(VPC3_GET_IND_NEW_GC_COMMAND())
        {
            user_isr_new_gc_command();
            VPC3_CON_IND_NEW_GC_COMMAND();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_NEW_GC_COMMAND == 1 )

    /*-----------------------------------------------------------------------*/
    /* IND_NEW_SSA_DATA                                                      */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_NEW_SSA_DATA == 1 )
        if(VPC3_GET_IND_NEW_SSA_DATA())
        {
            user_isr_new_ssa_data();
            dp_sys.vpc3_con = VPC3_FREE_SSA_BUF(); //release the Set Slave Address buffer
            VPC3_CON_IND_NEW_SSA_DATA();
        }
    #endif//#if( ISR_ENABLE_VPC3_INT_NEW_SSA_DATA == 1 )


    /*-----------------------------------------------------------------------*/
    /* IND_USER_TIMER_CLOCK                                                  */
    /*-----------------------------------------------------------------------*/
    #if( ISR_ENABLE_VPC3_INT_USER_TIMER_CLOCK == 1 )
        if(VPC3_GET_IND_USER_TIMER_CLOCK() )
        {
            user_isr_user_timer_clock();

            #ifdef DP_MSAC_C2
                msac_c2_timer_tick_10msec();
            #endif//#ifdef DP_MSAC_C2
            VPC3_CON_IND_USER_TIMER_CLOCK();
        }//if(VPC3_GET_IND_USER_TIMER_CLOCK() )
    #endif//#if( ISR_ENABLE_VPC3_INT_USER_TIMER_CLOCK == 1 )

    #ifdef DP_SUBSCRIBER
        /*-------------------------------------------------------------------*/
        /* IND_DXB_OUT                                                       */
        /*-------------------------------------------------------------------*/
        #if( ISR_ENABLE_VPC3_INT_DXB_OUT == 1 )
            if (VPC3_GET_IND_DXB_OUT())
            {
                user_isr_dxb_out();
                VPC3_CON_IND_DXB_OUT();
            }
        #endif//#if( ISR_ENABLE_VPC3_INT_DXB_OUT == 1 )

        /*-------------------------------------------------------------------*/
        /* IND_DXB_LINK_ERROR                                                */
        /*-------------------------------------------------------------------*/
        #if( ISR_ENABLE_VPC3_INT_DXB_LINK_ERROR == 1 )
            if (VPC3_GET_IND_DXB_LINK_ERROR())
            {
                user_isr_dxb_link_error();
                VPC3_CON_IND_DXB_LINK_ERROR();
            }
        #endif//#if( ISR_ENABLE_VPC3_INT_DXB_LINK_ERROR == 1 )
    #endif//#ifdef DP_SUBSCRIBER

    #ifdef DP_FDL
        /*-------------------------------------------------------------------*/
        /* IND_REQ_PDU                                                       */
        /*-------------------------------------------------------------------*/
        #if( ISR_ENABLE_VPC3_INT_FDL_IND == 1 )
            if( VPC3_GET_IND_FDL_IND() )
            {
                if( !VPC3_GET_IND_POLL_END_IND() )
                {
                    VPC3_CON_IND_FDL_IND();
                    fdl_mac_indication();
                }
            }
        #endif//#if( ISR_ENABLE_VPC3_INT_FDL_IND == 1 )

        /*-------------------------------------------------------------------*/
        /* IND_POLL_END                                                      */
        /*-------------------------------------------------------------------*/
        #if( ISR_ENABLE_VPC3_INT_POLL_END_IND == 1 )
            if( VPC3_GET_IND_POLL_END_IND() )
            {
                VPC3_CON_IND_POLL_END_IND();
                fdl_flc_indication();
            }
        #endif//#if( ISR_ENABLE_VPC3_INT_POLL_END_IND == 1 )
    #endif//#ifdef DP_FDL

    /*-----------------------------------------------------------------------*/
    /* reset interrupt line                                                  */
    /*-----------------------------------------------------------------------*/
    VPC3_SET_EOI();
}//void vpc3_isr( void )

/*****************************************************************************/
/*  Copyright (C) profichip GmbH 2004. Confidential.                         */
/*****************************************************************************/