/*****************************  Filename: serio.c ****************************/
/* ========================================================================= */
/*                                                                           */
/* 0000  000   000  00000 0  000  0   0 0 0000                               */
/* 0   0 0  0 0   0 0     0 0   0 0   0 0 0   0                              */
/* 0   0 0  0 0   0 0     0 0     0   0 0 0   0      Einsteinstrae 6        */
/* 0000  000  0   0 000   0 0     00000 0 0000       91074 Herzogenaurach    */
/* 0     00   0   0 0     0 0     0   0 0 0                                  */
/* 0     0 0  0   0 0     0 0   0 0   0 0 0          Tel: ++49-9132-744-200  */
/* 0     0  0  000  0     0  000  0   0 0 0    GmbH  Fax: ++49-9132-744-204  */
/*                                                                           */
/* ========================================================================= */
/*                                                                           */
/* Function:       serial function                                           */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Technical support:       P. Fredehorst                                    */
/*                          Tel. : ++49-9132/744-214                         */
/*                          Fax. :              -204                         */
/*                          eMail: pfredehorst@profichip.com                 */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/* contents:

  - function prototypes
  - data structures
  - internal functions

*/
/*****************************************************************************/
/* include hierarchy */

#include "..\..\dp_inc\platform.h"
#include "..\..\dp_inc\dp_inc.h"

#ifdef RS232_SERIO

/*---------------------------------------------------------------------------*/
/* local function prototypes                                                 */
/*---------------------------------------------------------------------------*/
void start_serial_communication ( void );
void set_baud                   ( UWORD baudrate );

/*---------------------------------------------------------------------------*/
/* global data definitions                                                   */
/*---------------------------------------------------------------------------*/
UBYTE       snd_buff [256]; // for RS232 sending
UBYTE       rec_buff [256]; // for RS232 receiving

UBYTE data  write_rec_ptr;  // Writepointer into receive buffer
UBYTE data  read_rec_ptr;   // Readpointer  from receive buffer
UBYTE data  rec_counter;    // Counter of byte in receive buffer
UBYTE data  write_snd_ptr;  // Writepointer into send buffer
UBYTE data  read_snd_ptr;   // Readpointer  from send buffer
UBYTE data  snd_counter;    // Counter of byte in send buffer
UBYTE       serial_state;   // shows state of serial line
bit         thr_empty;      // Transmitter reg. empty

/*---------------------------------------------------------------------------*/
/* local data definitions                                                    */
/*---------------------------------------------------------------------------*/
//static const UBYTE asccii[16] = {0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x41,0x42,0x43,0x44,0x45,0x46};
UBYTE asccii[16];

bit         parity;         // Save parity bit received

sbit CTS    = 0x93;
sbit RTS    = 0x94;


/*---------------------------------------------------------------------------*/
/* functions                                                                 */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   void print_string__(UBYTE *p)                                           */
/*                                                                           */
/*   function:     print string                                              */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void print_string__( UBYTE *p )
{
    while ( *p )
    {
        snd_buff [write_snd_ptr++] = *p++;
        snd_counter++;
    }
    start_serial_communication();
}//void print_string__( UBYTE *p )


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   UBYTE print_hexbyte__(UBYTE b)                                          */
/*                                                                           */
/*   function:     konverts integer to asccii                                */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void print_hexbyte__(UBYTE b)
{
    snd_buff [write_snd_ptr++] = asccii[(b >> 4)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(b & 0x0F)];
    snd_counter++;
    start_serial_communication();
}//void print_hexbyte__(UBYTE b)


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   UBYTE print_hexword__(UWORD w)                                          */
/*                                                                           */
/*   function:     konverts integer to asccii                                */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void print_hexword__(UWORD w)
{
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((w >> 12) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((w >> 8) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((w >> 4) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)(w & 0x000F)];
    snd_counter++;
    start_serial_communication();
}//void print_hexword__(UWORD w)


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   UBYTE print_hexlong__(ULONG dw)                                          */
/*                                                                           */
/*   function:     konverts integer to asccii                                */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void print_hexlong__(ULONG dw)
{
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 28) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 24) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 20) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 16) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 12) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 8) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)((dw >> 4) & 0x000F)];
    snd_counter++;
    snd_buff [write_snd_ptr++] = asccii[(UBYTE)(dw & 0x000F)];
    snd_counter++;
    start_serial_communication();
}//void print_hexlong__(ULONG dw)


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   void start_serial_communication(void)                                   */
/*                                                                           */
/*   function:     starts serial communication                               */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void start_serial_communication( void )
{
    //if(write_snd_ptr != read_snd_ptr)
    if(snd_counter > 0)
    {
        // disable interrupts
        DISABLE_ALL_INTERRUPT_BIT__;

        if( thr_empty )
        {
            thr_empty = FALSE;
            // set RTS handshake
            //RTS = 0;
            // transmit trigger via softw. interrupt
            TI = 1;
        }

        // enable interrupts
        ENABLE_ALL_INTERRUPT_BIT__;
    }
    return;
}//void start_serial_communication( void )


/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   void init_serio__(UWORD baudrate)                                        */
/*                                                                           */
/*   function:     initialize serial communication                           */
/*                                                                           */
/*---------------------------------------------------------------------------*/
#ifdef EvaBoard_AT89C5132
	
	#define Set_SMOD1()           	(PCON |= MSK_SMOD1)

void init_serio__( UWORD baudrate )
{
    memset(&snd_buff[0], 0, sizeof(snd_buff));
    memset(&rec_buff[0], 0, sizeof(snd_buff));

    write_rec_ptr = 0;      // Writepointer into receive buffer
    read_rec_ptr  = 0;      // Readpointer  from receive buffer
    rec_counter   = 0;      // Counter of byte in receive buffer
    write_snd_ptr = 0;      // Writepointer into send buffer
    read_snd_ptr  = 0;      // Readpointer  from send buffer
    snd_counter   = 0;      // Counter of byte in send buffer

	asccii[0] = 0x30;
	asccii[1] = 0x31;
	asccii[2] = 0x32;
	asccii[3] = 0x33;
	asccii[4] = 0x34;
	asccii[5] = 0x35;
	asccii[6] = 0x36;
	asccii[7] = 0x37;
	asccii[8] = 0x38;
	asccii[9] = 0x39;
	asccii[10] = 0x41;
	asccii[11] = 0x42;
	asccii[12] = 0x43;
	asccii[13] = 0x44;
	asccii[14] = 0x45;
	asccii[15] = 0x46;

    serial_state  = ON;     // shows state of serial line
    thr_empty     = TRUE;   // Transmitter reg. empty

    TXD = 1;
    RXD = 1;
	SCON = 0x50;					//uart mode: mode 1, 8bit, receive enable
   	BDRCON &=0xEC;               	//BRR=0; SRC=0;
   	BDRCON |=0x0E;               	//TBCK=1; RBCK=1; SPD=1

   	#ifdef AT89C5132_MODE_X2
   		switch( baudrate )
   		{
   			case 115200:
   			{
				Set_SMOD1();		//SMOD1 = 1
   				BRL = 245;
   				break;
   			}	

   			case 57600:
   			{
				Set_SMOD1();		//SMOD1 = 1
   				BRL = 234;
   				break;
   			}	

   			case 19200:
   			{
				Set_SMOD1();		//SMOD1 = 1
   				BRL = 191;
   				break;
   			}	

   			case 9600:
   			{
				Set_SMOD1();		//SMOD1 = 1
   				BRL = 126;
   				break;
   			}	

   			case 4800:
   			{
   				BRL = 126;
   				break;
   			}	

			case 38400:
   			default:
   			{
				Set_SMOD1();		//SMOD1 = 1
   				BRL = 223;
   				break;
   			}	
   		}//switch( baudrate )
   	#else
		Set_SMOD1();				//SMOD1 = 1
   		switch( baudrate )
   		{
   			case 57600:
   			{
   				BRL = 245;
   				break;
   			}	

   			case 19200:
   			{
   				BRL = 223;
   				break;
   			}	

   			case 9600:
   			{
   				BRL = 191;
   				break;
   			}	

   			case 4800:
   			{
   				BRL = 126;
   				break;
   			}	

			case 38400:
   			default:
   			{
   				BRL = 240;
   				break;
   			}	
   		}//switch( baudrate )
	#endif//#ifdef AT89C5132_MODE_X2

	REN=1;
    // clear interrupt flags
    TI = RI = 0;
	IPH0 |= MSK_ES;
	IPL0 |= MSK_ES;
 	ES = 1; 						//Enable serial interrupt
   	BDRCON |= 0x10;               	//Baud rate generator run
}//void init_serio__ (UWORD baudrate)
#endif//#ifdef EvaBoard_AT89C5132

#ifdef _C501
void init_serio__(UWORD baudrate)
{
    memset(&snd_buff[0], 0, sizeof(snd_buff));
    memset(&rec_buff[0], 0, sizeof(snd_buff));

    write_rec_ptr = 0;      // Writepointer into receive buffer
    read_rec_ptr  = 0;      // Readpointer  from receive buffer
    rec_counter   = 0;      // Counter of byte in receive buffer
    write_snd_ptr = 0;      // Writepointer into send buffer
    read_snd_ptr  = 0;      // Readpointer  from send buffer
    snd_counter   = 0;      // Counter of byte in send buffer

    serial_state  = ON;     // shows state of serial line
    thr_empty     = TRUE;   // Transmitter reg. empty

    // set timer 2 to baudrate generator
    // set default values to timer 2
    // set baudrate
    set_baud ( baudrate );

    T2CON = 0x00;
    // enable transmit and receive clock
    //T2IE = T2RSE = 1;
    RCLK = TCLK = 1;
    // start timer 2
    TR2 = 1;
    // serial line configuration
    // mode 3, 9bit UART
    SM0 = SM1 = 1;
    // port3 pins
    TXD = RXD = 1;
    // receive enable
    REN = 1;
    // clear interrupt flags
    TI = RI = 0;
    // enable serial interrupt
    ES = 1;
    // set high priority
    //PS = 1;
    CTS = 1 ;        // Define as Input pin
    RTS = 1 ;        // Do not want to send yet
}//void init_serio__ (UWORD baudrate)

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   void set_baud ( UWORD baudrate )                                        */
/*                                                                           */
/*   function:     set baudrate                                              */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void set_baud (UWORD baudrate)
{
    // set baudrate 24MHz
    switch(baudrate)
    {
        case 11520:
        {
            //set reload value
            RCAP2H = 0xFF;
            RCAP2L = 0xF8;
            break;
        }

        case 57600:
        {
            //set reload value
            RCAP2H = 0xFF;
            RCAP2L = 0xF3;
            break;
        }

        case 38400:
        {
            // set reload value
            RCAP2H = 0xFF;
            RCAP2L = 0xED;//0xEC;
            break;
        }

        case 19200:
        {
            // set reload value
            RCAP2H = 0xFF;
            RCAP2L = 0xD9;
            break;
        }

        case 9600:
        {
            // set reload value
            RCAP2H = 0xFF;
            RCAP2L = 0xB2;
            break;
        }

        default:
        {
            // Parameter Error
            // Unknown Baudrate
            break ;
        }

    } // end switch( baudrate )
    return;
}//void set_baud (UWORD baudrate)
#endif//#ifdef _C501

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   void ser_isr ( void ) interrupt                                         */
/*                                                                           */
/*   function:     interrupt routine                                         */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void ser_isr ( void ) interrupt IRQ_UART using 1          // Keil Compiler
//_interrupt(4) _using(1) void ser_isr (void)      // Tasking Compiler
{
    // disable interrupts
    DISABLE_ALL_INTERRUPT_BIT__;

    // send fifo empty
    if( _testbit_( TI ))
    {
        //if( read_snd_ptr == write_snd_ptr )
        if(snd_counter == 0)
        {
            // Nothing to send, set flag
            thr_empty = 1;
            // RTS handshake
            RTS = 1;
        }
        else
        {
            // something to send
            // Byte moved through the ACCU, to get the correct parity set
            ACC = snd_buff [read_snd_ptr];
            // even parity
            TB8 = P;
            // start send
            SBUF = ACC;
            // wraps automatically, since Byte
            read_snd_ptr++;
            snd_counter--;
        }
    }

    // receive fifo full
    if( _testbit_( RI ))
    {
        // read it in!
        rec_buff [write_rec_ptr] = SBUF;
        // Byte moved through the ACCU automatically, so
        // is the correct parity set
        parity = P;
        // here, check EVEN parity
        if( parity != RB8 )
        {

        }
        else
        {
            // clear error
        }

        if(serial_state == ON)
        {
            // wraps automatically since byte
            write_rec_ptr++;
            rec_counter++;
        }
        else
        {
            // clear serial buffer, serial communication is closed
            write_rec_ptr = read_rec_ptr;
            rec_counter = 0;
        }
    }

    // enable interrupts
    ENABLE_ALL_INTERRUPT_BIT__;
}

#endif//#ifdef RS232_SERIO

/*****************************************************************************/
/*  Copyright (C) profichip GmbH 2004. Confidential.                         */
/*****************************************************************************/