/**************************  Filename: dp_user.c  ****************************/
/* ========================================================================= */
/*                                                                           */
/* 0000  000   000  00000 0  000  0   0 0 0000                               */
/* 0   0 0  0 0   0 0     0 0   0 0   0 0 0   0                              */
/* 0   0 0  0 0   0 0     0 0     0   0 0 0   0      Einsteinstrae 6        */
/* 0000  000  0   0 000   0 0     00000 0 0000       91074 Herzogenaurach    */
/* 0     00   0   0 0     0 0     0   0 0 0                                  */
/* 0     0 0  0   0 0     0 0   0 0   0 0 0          Tel: ++49-9132-744-200  */
/* 0     0  0  000  0     0  000  0   0 0 0    GmbH  Fax: ++49-9132-744-204  */
/*                                                                           */
/* ========================================================================= */
/*                                                                           */
/* Function:      Demo for PROFICHIP Extension Board AT89C5132               */
/*                This example simulates a modular profibus device           */
/*                with max 6 Modules.                                       */
/*                                                                           */
/*				  Module  1: dip-switch PORT1                                */
/*				  Module  2: dip-switch PORT2                                */
/*				  Module  3: led PORT1                                       */
/*				  Module  4: led PORT2                                       */
/*				  Module  5: dio32                                           */
/*				  Module  6: counter                                         */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Hardware requirements: ProfiChip Evaluation Board AT89C5132  (PA006101)   */
/*                        ProfiChip Evaluation Board VPC3+/C    (PA006103)   */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/* memory:  0000H ... 7FFFH: RAM                                             */
/*          8000H ... 8FFFH: MPI12x                                          */
/*          9000H ... 9FFFH: Reserved                                        */
/*          A000H ... AFFFH: FPGA                                            */
/*          B000H ... BFFFH: RTC                                             */
/*          C000H ... CFFFH: LCD                                             */
/*          D000H ... DFFFH: I/O Port 0                                      */
/*          E000H ... EFFFH: I/O Port 1                                      */
/*          F000H ... FFFFH: I/O Port 2                                      */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Technical support:       P. Fredehorst                                    */
/*                          Tel. : ++49-9132/744-214                         */
/*                          Fax. :              -204                         */
/*                          eMail: pfredehorst@profichip.com                 */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/* contents:

  - function prototypes
  - data structures
  - internal functions

*/
/*****************************************************************************/
/* include hierarchy */

#include "..\..\dp_inc\platform.h"
#include "..\..\dp_inc\dp_inc.h"

/*---------------------------------------------------------------------------*/
/* defines, structures                                                       */
/*---------------------------------------------------------------------------*/
// -- defines for user alarm state
#define USER_AL_STATE_CLOSED		((UBYTE)0x00)
#define USER_AL_STATE_OPEN			((UBYTE)0x01)
// -- defines for user state
#define USER_STATE_CLEAR			((UBYTE)0x00)
#define USER_STATE_RUN				((UBYTE)0x01)
// -- defines for diagnostics
#define USER_TYPE_DPV0              ((UBYTE)0xFA)
#define USER_TYPE_PRM_OK            ((UBYTE)0xFB)
#define USER_TYPE_PRM_NOK           ((UBYTE)0xFC)
#define USER_TYPE_CFG_OK            ((UBYTE)0xFD)
#define USER_TYPE_CFG_NOK           ((UBYTE)0xFE)
#define USER_TYPE_APPL_RDY          ((UBYTE)0xFF)

// -- status message data (coded as device diagnosis) -------------------------
// -- Modulstatus -------------------------------------------------------------
typedef struct
{
    UBYTE   sign_len;       // fix to 0x07
    UBYTE   status_type;    // fix to 0x82
    UBYTE   slot_number;    // fix to 0x00
    UBYTE   specifier;      // fix to 0x00
    UBYTE   user_data[2];
} tStatusDiag;
#define sizeof_StatusDiag 6

tStatusDiag status_diag;

ROMCONST__ UBYTE DefStatusDiag[] = { 0x06, 0x82, 0x00, 0x00, 0xAA, 0x0A };

#define szUsrDiagnosticAlarmData    ((UBYTE)0x05)
#define szDiagnosticAlarm           (szAlarmHeader + szUsrDiagnosticAlarmData)

#define MaxAlarmLength 	((UBYTE)szUsrDiagnosticAlarmData)

typedef struct
{
    UBYTE 						state;
    UBYTE 						user_diag_active;
    UWORD 						old_diag;
    UBYTE 						user_diag[DIAG_BUFSIZE];
	UWORD						event;
	UBYTE 						application_ready;
	CFG_STRUCT 					real_cfg;	//real cfg data

    struct                  	//alarm state machine data
    {
        UBYTE 					state;
		UBYTE 					buffer_used[ALARM_MAX_FIFO];
		UBYTE 					user_alarm_data[ALARM_MAX_FIFO][MaxAlarmLength];
		ALARM_STATUS_PDU		alarm_buffers[ALARM_MAX_FIFO];
    }al;	
}USER_STRUC;

/*---------------------------------------------------------------------------*/
/* global user data definitions                                              */
/*---------------------------------------------------------------------------*/
VPC3_STRUC_ERRCB                vpc3_errcb; // error structure
USER_STRUC                      user;   	// user system structure

/*---------------------------------------------------------------------------*/
/* defines, structures and variables for our demo application                */
/*---------------------------------------------------------------------------*/
#define MaxModule ((UBYTE)0x06)

//default configuration data for startup
ROMCONST__ UBYTE DefCfg[24] = { 	0x42,0x00,0x00,0x01,	//DI8 (PORT 1) 
									0x42,0x00,0x00,0x02,	//DI8 (PORT 2)
                            		0x82,0x00,0x00,0x03,	//DO8 (PORT 1) 
                            		0x82,0x00,0x00,0x04,	//DO8 (PORT 2) 
                            		0xC1,0x03,0x03,0x05,	//DIO32  
                            		0xC1,0x01,0x01,0x06, 	//Counter  16bit 
                        	  };

//counter module
static UWORD 	counter_value;
static UBYTE    counter_timebase;
static UBYTE    counter_upper_limit;
static UBYTE    counter_lower_limit;

typedef struct
{
    UBYTE alarm;
    UBYTE mode;
    UBYTE time_base;
    UWORD upper_limit;
    UWORD lower_limit;
} STRUC_PRM_COUNTER;

typedef struct
{
	UBYTE do8[2];
	UBYTE dio32[4];
	UBYTE counter[2];
} STRUC_OUTPUT;

typedef struct
{
	UBYTE di8[2];
	UBYTE dio32[4];
	UBYTE counter[2];
} STRUC_INPUT;

typedef struct
{
	UBYTE 				NrOfModules;
	STRUC_INPUT			input;
	STRUC_OUTPUT		output;
	STRUC_PRM_COUNTER 	prm;
} STRUC_SYSTEM;

STRUC_SYSTEM sSystem;


/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* user_alarm_init function                                                 */
/*--------------------------------------------------------------------------*/
void user_alarm_init(void)
{
    memset( &user.al.buffer_used[0], 0, ALARM_MAX_FIFO );
}//void user_alarm_init(void)

/*--------------------------------------------------------------------------*/
/* user_alarm_alloc function                                                */
/*--------------------------------------------------------------------------*/
ALARM_STATUS_PDU_PTR user_alarm_alloc( void )
{
ALARM_STATUS_PDU_PTR ptr;
UBYTE                i;
    
    ptr = NULL;
    for( i = 0; i < ALARM_MAX_FIFO; i++ )
    {
        if(user.al.buffer_used[i] == FALSE)
        {
            ptr = &(user.al.alarm_buffers[i]);
            user.al.buffer_used[i] = TRUE;
            ptr->header        = 0;
            ptr->type          = 0;
            ptr->slot          = 0;
            ptr->spec          = 0;
            ptr->user_data_len = 0;
            ptr->user_data_ptr = &(user.al.user_alarm_data[i][0]);
            break;
        }
    }//for( i = 0; i < ALARM_MAX_FIFO; i++ )
    return ptr;
}//ALARM_STATUS_PDU_PTR user_alarm_alloc( void )

/*--------------------------------------------------------------------------*/
/* user_free_alarm_buffer function                                          */
/*--------------------------------------------------------------------------*/
void user_free_alarm_buffer(ALARM_STATUS_PDU_PTR alarm_ptr)
{
ALARM_STATUS_PDU_PTR ptr;
UBYTE                i;

    for( i = 0; i <= ALARM_MAX_FIFO; i++ )
    {
        if( user.al.buffer_used[i] == TRUE )
        {
        	ptr = &(user.al.alarm_buffers[i]);
            if( ptr == alarm_ptr )
            {
            	user.al.buffer_used[i] = FALSE;
            	break;
        	}//if( ptr == alarm_ptr )
        }//if( user.al.buffer_used[i] == FALSE )	
    }//for( i = 0; i <= ALARM_MAX_FIFO; i++ )
}//void user_free_alarm_buffer(ALARM_STATUS_PDU_PTR ptr)

/*---------------------------------------------------------------------------*/
/* function: user_alarm ( is also called from alarm state machine !!!! )     */
/*---------------------------------------------------------------------------*/
UBYTE user_alarm( UBYTE alarm_type, UBYTE seq_nr, ALARM_STATUS_PDU_PTR alarm_ptr, UBYTE check_diag_flag )
{
ROMCONST__ UBYTE chn_diag_5[3] = { 0x84, 0xC1, 0x82 };	//module 5
ROMCONST__ UBYTE chn_diag_6[3] = { 0x85, 0xC1, 0x82 };	//module 6

VPC3_UNION_DIAG_PTR tmp_diag;
UBYTE 				ret_value;
UBYTE               ext_diag;
UBYTE               len_diag;
UBYTE 				error;
UWORD 				diag;

    ret_value = 0x00;

	diag = ( (UWORD)(alarm_type << 8) | ((UWORD)seq_nr) );
		
    //don't send diagnostic twice!
    if(	   ( diag  != user.old_diag         )
    	&& ( FALSE == user.user_diag_active )
      )	
    {
        memset( &user.user_diag[0], 0x00, sizeof( user.user_diag ) );
        tmp_diag.byte_ptr = user.user_diag;

        ext_diag = 0x00;
        len_diag = 0x00;
        
        switch( alarm_type )
        {
            case USER_TYPE_CFG_OK:
            {
                ext_diag = STAT_DIAG_SET;
                break;
            }//case USER_TYPE_CFG_OK:

            case USER_TYPE_CFG_NOK:
            {
                //modul_status diagnose
                memcpy( tmp_diag.byte_ptr, &status_diag.sign_len, sizeof_StatusDiag );

                ext_diag = EXT_DIAG_SET;
                len_diag = sizeof_StatusDiag;
                break;
            }//case USER_TYPE_APPL_RDY:

            case USER_TYPE_DPV0:
            {
            	if( alarm_ptr->type == STATUS_MESSAGE )
            	{
            		//diagnostic alarm
                    if( (alarm_ptr->spec & SPEC_MASK) == SPEC_APPEARS )
                    {
            			//only an example
            			//add here for the appearing diagnostic alarm the channel related diagnostic
                		ext_diag = EXT_DIAG_SET;
                		if( alarm_ptr->slot == 0x05 )
                		{	
                			//module 5
                			memcpy( tmp_diag.byte_ptr+len_diag, (UBYTE *)&chn_diag_5[0], 3 );
                			len_diag += 3;
                		}
                		else	
                		{	
                			//module 6
                			memcpy( tmp_diag.byte_ptr+len_diag, (UBYTE *)&chn_diag_6[0], 3 );
                			len_diag += 3;
                		}	
                    }

                	memcpy( tmp_diag.byte_ptr+len_diag, (UBYTE *)&alarm_ptr->header, szAlarmHeader );
                	len_diag += szAlarmHeader;
                	memcpy( tmp_diag.byte_ptr+len_diag, alarm_ptr->user_data_ptr, szUsrDiagnosticAlarmData );
                	len_diag += szUsrDiagnosticAlarmData;
            	}
            	break;
            }
            
        	case USER_TYPE_APPL_RDY:
            case USER_TYPE_PRM_NOK:
            case USER_TYPE_PRM_OK:
            default:
            {
                ext_diag = 0x00;
                len_diag = 0x00;
                break;
            }//default:
        }//switch(user[vpc3_channel].diag_byte)

        user.user_diag_active = TRUE;
            
        error = set_diagnosis( tmp_diag, len_diag, ext_diag, check_diag_flag  );

        if( error == DP_OK )
        {
            user.old_diag = diag;
            if( alarm_type >= USER_TYPE_PRM_OK )
            {
                user.user_diag_active = FALSE;
            }//if( alarm_type >= USER_TYPE_PRM_OK )
            ret_value = DP_OK;
        }//if( error == DP_OK )
        else
        {
        	user.user_diag_active = FALSE;
			
			#ifdef RS232_SERIO
            	print_string("DIAG_DP_Error: ");
            	print_hexbyte(error);
            	print_hexbyte(alarm_type);
			#endif//#ifdef RS232_SERIO
            ret_value = error;
        }//else of if( error == DP_OK )
    }//if(	   ( diag  != user.old_diag         ) ...

    return ret_value;
}//UBYTE user_alarm( UBYTE diag_event, UBYTE crc_byte, ALARM_STATUS_PDU_PTR alarm_ptr )

UBYTE user_demo_diagnostic( UBYTE diag_nr, UBYTE module_nr )
{
ROMCONST__ UBYTE diagnostic_alarm_appears[5]    = { 0x01,0x07,0x0F,0x67,0x33 };
ROMCONST__ UBYTE diagnostic_alarm_disappears[5] = { 0x01,0x00,0x0F,0x00,0x00 };

static UBYTE diagnostic_alarm_seq_nr;

ALARM_STATUS_PDU_PTR p_alarm;
UBYTE ret_value;
UBYTE callback;

	ret_value = TRUE;
	
	p_alarm = user_alarm_alloc();
    if( p_alarm != NULL )
    {
    	callback = FALSE;
    	switch( diag_nr )
    	{
    		case 1:
    		case 3:
    		{
    			diagnostic_alarm_seq_nr = (diagnostic_alarm_seq_nr + 1) & (MAX_SEQ_NR - 1);
                p_alarm->header         = szDiagnosticAlarm;
                p_alarm->type           = STATUS_MESSAGE;
                p_alarm->slot           = module_nr;
                p_alarm->spec           = SPEC_APPEARS + (diagnostic_alarm_seq_nr << SPEC_SEQ_START);
                p_alarm->user_data_len  = szUsrDiagnosticAlarmData;
                memcpy( p_alarm->user_data_ptr, &diagnostic_alarm_appears[0], szUsrDiagnosticAlarmData );
                //The alarm state machine calls the function user_alarm again.
                //The user can can add to the alarm e.g. module status or channel related diagnostic.
                callback = TRUE;	
    			break;
    		}
    		
    		case 2:
    		case 4:
    		{
    			diagnostic_alarm_seq_nr = (diagnostic_alarm_seq_nr + 1) & (MAX_SEQ_NR - 1);
                p_alarm->header         = szDiagnosticAlarm;
                p_alarm->type           = STATUS_MESSAGE;
                p_alarm->slot           = module_nr;
                p_alarm->spec           = SPEC_DISAPPEARS + (diagnostic_alarm_seq_nr << SPEC_SEQ_START);
                p_alarm->user_data_len  = szUsrDiagnosticAlarmData;
                memcpy( p_alarm->user_data_ptr, &diagnostic_alarm_disappears[0], szUsrDiagnosticAlarmData );
    			break;
    		}

            default:
            {
            	break;
        	}
        }//switch( diag_nr )     	
            
       	user_alarm( USER_TYPE_DPV0, p_alarm->spec, p_alarm, FALSE );
        user_free_alarm_buffer( p_alarm );
    }//if( p_alarm != NULL )

	return ret_value;
}//UBYTE user_demo_diagnostic( UBYTE diag_nr )	

void read_input_data( void )
{
VPC3_UNSIGNED8_PTR  input_buf_ptr; 	// pointer to input buffer

    // write DIN data to VPC3
    if(input_buf_ptr = vpc3_get_dinbufptr ())            // get pointer to DIN data
    {
        //todo:
        //handle here data (slave --> master) 
		
       	memcpy( input_buf_ptr, &sSystem.input.di8[0], dp_sys.inp_data_len );

        // the user makes a new Din-Buffer available in the state N
        dp_sys.vpc3_con = VPC3_INPUT_UPDATE();
    }//if(input_buf_ptr = vpc3_get_dinbufptr ())
}//void read_input_data( void )	


void application_ready( void )
{
	#ifdef DPV1_IM_SUPP
		user.awImIndex[ 0x02 ] = 0x0000;
	#endif//#ifdef DPV1_IM_SUPP

	//todo
	//make here your own initialization	
		
	//read input data
    read_input_data();
    //reset Diag.Stat 
    user_alarm( USER_TYPE_APPL_RDY, 0, (ALARM_STATUS_PDU_PTR) NULL_PTR, FALSE );
    user.application_ready = TRUE;
}//void application_ready( void )	


/*---------------------------------------------------------------------------*/
/* user main function                                                        */
/*---------------------------------------------------------------------------*/
void user_main(void)
{
VPC3_UNSIGNED8_PTR  output_buf_ptr; // pointer to output buffer
DP_ERROR_CODE		error;
UBYTE           	outp_state; 	// state of output data
#ifdef EvaBoard_AT89C5132
	UBYTE 			bDipSwitch5;
	UBYTE 			bOldDipSwitch5;
	UBYTE			bDiagStateModule5;
	UBYTE			bDiagStateModule6;
#endif//#ifdef EvaBoard_AT89C5132

    /*-----------------------------------------------------------------------*/
    /* init user data                                                        */
    /*-----------------------------------------------------------------------*/
	memset( &user, 0, sizeof(user) );

    /* TRUE  deactivates diagnosis handling ! */
    /* FALSE activates   diagnosis handling ! */
    user.user_diag_active  = FALSE;
	user.application_ready = FALSE;
    user.state = USER_STATE_CLEAR;

    /*-----------------------------------------------------------------------*/
    /* init System                                                           */
    /*-----------------------------------------------------------------------*/
	memset( &sSystem, 0, sizeof(STRUC_SYSTEM) );
	sSystem.NrOfModules = MaxModule;
	
    /*-----------------------------------------------------------------------*/
    /* initialize VPC3                                                       */
    /*-----------------------------------------------------------------------*/

    //todo: 
    //insert your real configuration data here
    user.real_cfg.length = (MaxModule*4); // length of configuration data
	memcpy( &user.real_cfg.cfg_data[0], &DefCfg[0], user.real_cfg.length );

	#ifdef EvaBoard_AT89C5132
    	error = vpc3_initialization( (*READ_PORT0 & 0x7F), user.real_cfg ); 	// address of slave; PORT0
	#else
    	error = vpc3_initialization( DP_ADDR, user.real_cfg );					// address of slave
	#endif//#ifdef EvaBoard_AT89C5132
    
    if( DP_OK == error )
    {
    	// set microcontroller interrupt to falling edge
    	SET_FALLING_EDGE__;
    	// enable microcontroller interrupt
    	ENABLE_VPC3_INT__;

        //todo: before startup the VPC3+, make here your own initialzations 
        
        START_VPC3();
    }
    else
    {
        vpc3_errcb.error_code = error;
        fatal_error( _DP_USER, __LINE__, &vpc3_errcb );
    }//if( DP_OK == error )

    /*-----------------------------------------------------------------------*/
    /* endless demo operation loop                                           */
    /*-----------------------------------------------------------------------*/
    while( 1 )
    {
        /*-------------------------------------------------------------------*/
        /* trigger watchdogs                                                 */
        /*-------------------------------------------------------------------*/
        // toggle user watchdog
        VPC3_RESET_USER_WD();   // toggle user watchdog

		#ifdef RS232_SERIO
        	if(rec_counter > 0)
        	{
				DISABLE_ALL_INTERRUPT_BIT__;
				
            	print_serial_inputs();
            	
            	ENABLE_ALL_INTERRUPT_BIT__;
        	}
		#endif//#ifdef RS232_SERIO

        /*-------------------------------------------------------------------*/
        /* internal state machine                                            */
        /*-------------------------------------------------------------------*/
        if( user.state == USER_STATE_CLEAR )
        {
            // clear data
			memset( &sSystem.output.do8[0] , 0, 9 );
			memset( &sSystem.input.di8[0]  , 0, 9 );

			#ifdef EvaBoard_AT89C5132
				bDiagStateModule5  = FALSE;
				bDiagStateModule6  = FALSE;

            	*WRITE_PORT1 = 0x00;
            	*WRITE_PORT2 = 0x00;
			#endif//#ifdef EvaBoard_AT89C5132

            user.state = USER_STATE_RUN;
        }//if( user.state == USER_STATE_CLEAR )

        /*-------------------------------------------------------------------*/
        /* dpv1 statemachines                                                */
        /*-------------------------------------------------------------------*/
        vpc3_dpv1_statemachine();

        /*-------------------------------------------------------------------*/
        /* VPC3+ DP-state                                                    */
        /*-------------------------------------------------------------------*/
        switch(VPC3_GET_DP_STATE())
        {
            case WAIT_PRM:
            case WAIT_CFG:
            case DP_ERROR:
            {
                // set LED's
                CLR_LED_YLW__;
                SET_LED_RED__;
                
                if( user.event & VPC3_EV_DX_OUT )
				{
					user.event &= ~VPC3_EV_DX_OUT;       // clear event
				}		
                break;
            }

            case DATA_EX:
            {
                // set LED's
                SET_LED_YLW__;
                CLR_LED_RED__;
				
				if(    ( user.application_ready == TRUE )
					&& ( user.state == USER_STATE_RUN   )
				  )
				{
					/*-------------------------------------------------------------------*/
        			/* profibus output                                                   */
        			/*-------------------------------------------------------------------*/
					if( user.event & VPC3_EV_DX_OUT )
					{
						user.event &= ~VPC3_EV_DX_OUT;       // clear event
        			    if( output_buf_ptr = vpc3_get_doutbufptr (&outp_state) )
        			    {
        			    	//todo:
        			    	//handle here data (master --> slave) 
                			memcpy( &sSystem.output.do8[0], output_buf_ptr, dp_sys.outp_data_len);
                			
        					/*-----------------------------------------------------------*/
        					/* user application                                          */
        					/*-----------------------------------------------------------*/
							#ifdef EvaBoard_AT89C5132
            					*WRITE_PORT1 = sSystem.output.do8[0];
            					*WRITE_PORT2 = sSystem.output.do8[1];
							#endif//#ifdef EvaBoard_AT89C5132

							if(	   ( sSystem.input.dio32[0] != sSystem.output.dio32[0] )
					    		|| ( sSystem.input.dio32[1] != sSystem.output.dio32[1] )
					    		|| ( sSystem.input.dio32[2] != sSystem.output.dio32[2] )
					    		|| ( sSystem.input.dio32[3] != sSystem.output.dio32[3] )
					  		  )  
							{
								sSystem.input.dio32[0] = sSystem.output.dio32[0];
								sSystem.input.dio32[1] = sSystem.output.dio32[1];
								sSystem.input.dio32[2] = sSystem.output.dio32[2];
								sSystem.input.dio32[3] = sSystem.output.dio32[3];
								user.event |= VPC3_EV_NEW_INPUT_DATA;
							}
							
							//counter module
							//clear counter
							if( sSystem.output.counter[0] & 0x02 )
							{
								sSystem.input.counter[0] = 0x00;
								sSystem.input.counter[1] = 0x00;
								counter_value			= 0x0000;
								user.event |= VPC3_EV_NEW_INPUT_DATA;
							}
                		}//if( user.output_buf_ptr = vpc3_get_doutbufptr (&outp_state) ) 	
        			}//if( user.event & VPC3_EV_DX_OUT )
        			
        			/*-------------------------------------------------------------------*/
        			/* user application                                                  */
        			/*-------------------------------------------------------------------*/
					#ifdef EvaBoard_AT89C5132
						if( sSystem.input.di8[0] != *READ_PORT1 )
						{
							sSystem.input.di8[0] = *READ_PORT1;
							user.event |= VPC3_EV_NEW_INPUT_DATA;
						}
						
						if( sSystem.input.di8[1] != *READ_PORT2 )
						{
							sSystem.input.di8[1] = *READ_PORT2;
							user.event |= VPC3_EV_NEW_INPUT_DATA;
						}
					
						//demo for diagnostic
						bDipSwitch5 = *READ_PORT2;
						if( bDipSwitch5 != bOldDipSwitch5 )
						{
							bOldDipSwitch5 = bDipSwitch5;
							
							#ifdef RS232_SERIO
								print_string("DipSwitchS5: ");
								print_hexbyte(bDipSwitch5);
							#endif//#ifdef RS232_SERIO
							
							//set alarm
							if( ( bDipSwitch5 & 0x10 ) && ( bDiagStateModule5 == FALSE ) )
							{
								bDiagStateModule5 = TRUE;
								user_demo_diagnostic( 1, 5 );
							}//if( ( bDipSwitch5 & 0x10 ) && ( bDiagStateModule5 == FALSE ) )
                    		
                    		//clear alarm
							if( !( bDipSwitch5 & 0x10 ) && ( bDiagStateModule5 == TRUE ) )
							{
								bDiagStateModule5 = FALSE;
								user_demo_diagnostic( 2, 5 );
							}//if( !( bDipSwitch5 & 0x10 ) && ( bDiagStateModule5 == TRUE ) )
                    		
							//set alarm
							if( ( bDipSwitch5 & 0x20 ) && ( bDiagStateModule6 == FALSE ) )
							{
								bDiagStateModule6 = TRUE;
								user_demo_diagnostic( 3, 6 );
							}//if( ( bDipSwitch5 & 0x20 ) && ( bDiagStateModule6 == FALSE ) )
                    		
                    		//clear alarm
							if( !( bDipSwitch5 & 0x20 ) && ( bDiagStateModule6 == TRUE ) )
							{
								bDiagStateModule6 = FALSE;
								user_demo_diagnostic( 4, 6 );
							}//if( !( bDipSwitch5 & 0x20 ) && ( bDiagStateModule6 == TRUE ) )
						}//if( bDipSwitch5 != bOldDipSwitch5 )	
					#endif//#ifdef EvaBoard_AT89C5132
					
        			/*-------------------------------------------------------------------*/
        			/* profibus input                                                    */
        			/*-------------------------------------------------------------------*/
					if( user.event & VPC3_EV_NEW_INPUT_DATA )
					{
						read_input_data();
						user.event &= ~VPC3_EV_NEW_INPUT_DATA;
					}
				}//if(    ( user.application_ready == TRUE ) ...
				
                break;
            }//case DATA_EX:

            default:
            {
                vpc3_errcb.error_code = VPC3_GET_DP_STATE();
                fatal_error( _DP_USER, __LINE__, &vpc3_errcb );
                break;
            }
        }//switch(VPC3_GET_DP_STATE())

        /*-------------------------------------------------------------------*/
        /* handle here profibus interrupt events                             */
        /*-------------------------------------------------------------------*/
    	if( user.event & VPC3_EV_NEW_CFG_DATA )
    	{
        	application_ready();
        	user.event &= ~VPC3_EV_NEW_CFG_DATA;    // clear event
    	}
    }//while(1)
}//void user_main(void)

/*---------------------------------------------------------------------------*/
/* delay                                                                     */
/*---------------------------------------------------------------------------*/
void delay( void )
{
UBYTE i,j;

    for(i = 0; i < 255; i++)
    {
        for(j = 0; j < 255; j++);
    }
}

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   function:     fatal_error                                               */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void fatal_error( DP_ERROR_FILE file, UWORD line, VPC3_ERRCB_PTR errcb_ptr)
{
	#ifdef RS232_SERIO
    	do
    	{
    	    // wait!
    	}
    	while( snd_counter > 80);
    	
    	print_string("\r\nFATAL_ERROR:");
    	print_string("\r\nFile: ");
    	print_hexbyte(file);
    	print_string("\r\nLine: ");
    	print_hexword(line);
    	print_string("\r\nFunction: ");
    	print_hexbyte(errcb_ptr->function);
    	print_string("\r\nError_Code: ");
    	print_hexbyte(errcb_ptr->error_code);
    	print_string("\r\nDetail: ");
    	print_hexbyte(errcb_ptr->detail);
    	print_string("\r\ncn_id: ");
    	print_hexbyte(errcb_ptr->cn_id);
	#endif//#ifdef RS232_SERIO

	#ifdef EvaBoard_AT89C5132
    	*WRITE_PORT0 = errcb_ptr->error_code;
    	*WRITE_PORT1 = file;
    	*WRITE_PORT2 = (UBYTE)line;
	#endif//#ifdef EvaBoard_AT89C5132

    SET_LED_YLW__;
    SET_LED_RED__;

    while(1)
    {
		#ifdef RS232_SERIO
        	if(rec_counter > 0)
        	{
            	print_serial_inputs();
        	}
		#endif//#ifdef RS232_SERIO

        TOGGLE_LED_RED__;
        TOGGLE_LED_YLW__;

        delay();
    }//while(1)
}//void fatal_error( DP_ERROR_FILE file, UWORD line, VPC3_ERRCB_PTR errcb_ptr)


/*---------------------------------------------------------------------------*/
/* function: user_isr_new_prm_data                                           */
/*                                                                           */
/* todo  : check the prm-data                                                */
/* return: DP_OK  - prm-data OK                                              */
/*         DP_NOK - prm-data not OK                                          */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_PRM_DATA == 1 )
UBYTE user_chk_new_prm_data( VPC3_UNSIGNED8_PTR prm_ptr, UBYTE len )
{
typedef struct
{
    UBYTE len;
    UBYTE slot_nr;
    UBYTE user_data;
}STRUC_MODULE_PRM_BLOCK;
#define VPC3_STRUC_MODULE_PRM_BLOCK_PTR  STRUC_MODULE_PRM_BLOCK PTR_ATTR *

VPC3_STRUC_MODULE_PRM_BLOCK_PTR module_block;
VPC3_STRUC_PRM_PTR 				prm_data;
UBYTE              				ret_value;

    ret_value = DP_OK;

    if( len == 0x13 )
    {
        prm_data = (VPC3_STRUC_PRM_PTR)prm_ptr;

        //DP-Mode
		if(    (( prm_data->dpv1_status_1 & 0x98 ) != 0x00 )
			|| (( prm_data->dpv1_status_2 & 0xFF ) != 0x00 )
			|| (( prm_data->dpv1_status_3 & 0xFF ) != 0x00 )
		  )	
		{
           	ret_value = DP_PRM_DPV1_STATUS;
        }//if(    (( prm_data->dpv1_status_1 & 0x18 ) != 0x00 )

        if( ret_value == DP_OK )
        {
			//user parameter data
            module_block = (VPC3_STRUC_MODULE_PRM_BLOCK_PTR)&prm_data->user_data;
			
			if( ( module_block->len + PRM_LEN_DPV1 ) == 0x13 )
			{
				//copy counter parameter data					
				memcpy( &sSystem.prm, &module_block->user_data, module_block->len-2 );
				
				//check parameter of counter module
            	if(    ( sSystem.prm.alarm     &  0xFC )		//check alarm
            		|| ( sSystem.prm.mode      >  0x01 )		//check counter mode
            		|| ( sSystem.prm.time_base >  0x05 )		//check counter timebase
             	  )		
            	{
            		//error
        			ret_value = DP_PRM_ENTRY_ERROR;
            	}//if(    ( sSystem.prm.alarm     &  0xFC ) ... 	
			}
			else
			{
				//length error
        		ret_value = DP_PRM_LEN_ERROR;
			}
        }//if( ret_value == DP_OK )
    }//if( len == 0x13 )
    else
    {
        ret_value = DP_PRM_LEN_ERROR;
    }//else of if( len == 0x13 )

    if( ( VPC3_GET_DP_STATE() == DATA_EX ) && ( ret_value == DP_OK ) )
    {
        //don't send diagnostic here
    }//if( ( VPC3_GET_DP_STATE() == DATA_EX ) && ( ret_value == DP_OK ) )
    else
    {
        user_alarm( USER_TYPE_PRM_OK, 0, (ALARM_STATUS_PDU_PTR) NULL_PTR, FALSE );
    }//else of if( ( VPC3_GET_DP_STATE() == DATA_EX ) && ( ret_value == DP_OK ) )

	return ret_value;	
}//UBYTE user_chk_new_prm_data(VPC3_UNSIGNED8_PTR prm_ptr, UBYTE len)
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_PRM_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_ext_prm_data                                       */
/*                                                                           */
/* todo  : check the prm-data                                                */
/* return: DP_OK  - prm-data OK                                              */
/*         DP_NOK - prm-data not OK                                          */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_EXT_PRM_DATA == 1 )
UBYTE user_chk_new_ext_prm_data( VPC3_STRUC_PRM_BLOCK_PTR prmblock_ptr, UBYTE len )
{
UBYTE ret_value;

	ret_value = DP_OK;
	len = len;
	
	if(	   ( prmblock_ptr->command == PBC_USER_PRM_DATA )
		&& ( prmblock_ptr->len     == 0x0C )
		&& ( prmblock_ptr->slot_nr == 0x00 )
		&& ( prmblock_ptr->index   == 0x00 )
	  )	
	{
		//copy counter parameter data					
		memcpy( &sSystem.prm, &prmblock_ptr->user_data+1, 7 );

		//check parameter of counter module
        if(    ( sSystem.prm.alarm     &  0xFC )		//check alarm
           	|| ( sSystem.prm.mode      >  0x01 )		//check counter mode
           	|| ( sSystem.prm.time_base >  0x05 )		//check counter timebase
          )		
        {
        	//error
        	ret_value = DP_PRM_ENTRY_ERROR;
        }//if(    ( counter.prm.len != 9 )  ... 	
	}
	else
	{
		ret_value = DP_PRM_BLOCK_ERROR;
	}
		
	return ret_value;
}
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_EXT_PRM_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_cfg_data 											 */
/*									                                         */
/* todo  : check cfg data   		                                         */
/*									                                         */
/* return:	VPC3_CFG_OK      0												 */		
/*			VPC3_CFG_FAULT   1												 */	
/*			VPC3_CFG_UPDATE  2								                 */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_CFG_DATA == 1 )
UBYTE user_chk_new_cfg_data(void)
{
VPC3_UNSIGNED8_PTR  cfg_ptr;   // pointer check config buffer
UBYTE               nr_of_check_modules;
UBYTE               nr_of_modules;
UBYTE               real_cfg_len;
UBYTE               ret_value;
UBYTE               cfg_len;
UBYTE               i;

    cfg_ptr  = VPC3_GET_CFG_BUF_PTR();

    real_cfg_len = sSystem.NrOfModules * 4;	//4 cfg-byte per module
    cfg_len = VPC3_GET_CFG_LEN();

    //build modulstatus diagnose
    memcpy( &status_diag.sign_len, &DefStatusDiag[0], sizeof_StatusDiag );
    
    nr_of_check_modules = sSystem.NrOfModules;
            
    if( cfg_len != real_cfg_len )
    {
        if( cfg_len < real_cfg_len )
        {
            nr_of_check_modules = ( cfg_len / 4 );
        }//if( cfg_len < etm_cfg_len )
        else
        {
            nr_of_modules = cfg_len / 5;
            nr_of_modules += ( cfg_len % 5 ) ? 1 : 0 ;

            for( i = nr_of_check_modules; i < nr_of_modules; i++)
            {
                status_diag.user_data[ i/4 ] |= (0x03 << ((i%4)*2));
            }//for( i = 0; i < nr_of_modules; i++)
        }//if( cfg_len > real_cfg_len )

        ret_value = VPC3_CFG_FAULT;
    }//if( cfg_len != real_cfg_len )
    else
    {
        ret_value = VPC3_CFG_OK;
    }//if( cfg_len != real_cfg_len )

    for( i = 0; i < nr_of_check_modules; i++)
    {
        if( memcmp( &user.real_cfg.cfg_data[i*4], cfg_ptr, 0x04 ) == 0 )
        {
            status_diag.user_data[ i/4 ] &= ~(0x02 << ((i%4)*2));
        }
        else
        {
            ret_value = VPC3_CFG_FAULT;
        }

        cfg_ptr += 4;
    }//for( i = 0; i < nr_of_check_modules; i++)

    if( ret_value == VPC3_CFG_OK )
    {
        user_alarm( USER_TYPE_CFG_OK, 0, (ALARM_STATUS_PDU_PTR) NULL_PTR, FALSE );
        user.event |= VPC3_EV_NEW_CFG_DATA;
    }//if( ret_value == VPC3_CFG_UPDATE )
    else
    {
        user_alarm( USER_TYPE_CFG_NOK, status_diag.user_data[ 0 ], (ALARM_STATUS_PDU_PTR) NULL_PTR, FALSE );
    }//else of if( ret_value == VPC3_CFG_UPDATE )
    user_alarm_init();
	return ret_value;		
}//UBYTE user_chk_new_cfg_data(void)
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_CFG_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_go_leave_data_ex                                       */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_GO_LEAVE_DATA_EX == 1 )
void user_isr_go_leave_data_ex( void )
{
	if( VPC3_GET_DP_STATE() != DATA_EX )
   	{
        user.state = USER_STATE_CLEAR;
   		user.application_ready = FALSE;
    }//if( VPC3_GET_DP_STATE() != DATA_EX )		
}//void user_isr_go_leave_data_ex( void )
#endif//#if( ISR_ENABLE_VPC3_INT_GO_LEAVE_DATA_EX == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_dx_out                                                 */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_DX_OUT == 1 )
void user_isr_dx_out( void )
{
	user.event |= VPC3_EV_DX_OUT;
}	
#endif//#if( ISR_ENABLE_VPC3_INT_DX_OUT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_diag_buf_changed                                       */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_DIAG_BUF_CHANGED == 1 )
void user_isr_diag_buf_changed( void )
{
    // diagnosis buffer has been changed
    user.user_diag_active = FALSE;
    // Fetch new diagnosis buffer
    dp_sys.diag_buf_ptr.byte_ptr = vpc3_get_diagbufptr();
}//void user_isr_diag_buf_changed( void )
#endif//#if( ISR_ENABLE_VPC3_INT_DIAG_BUF_CHANGED == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_wd_timeout                                         */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_WD_DP_TIMEOUT == 1 )
void user_isr_new_wd_dp_timeout( void )
{
	reset_diagnostic_buffer();
}//void user_isr_new_wd_dp_timeout( void )
#endif//#if( ISR_ENABLE_VPC3_INT_WD_DP_TIMEOUT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_mac_reset                                              */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_MAC_RESET == 1 )
void user_isr_mac_reset( void )
{
	//not used in our application
}	
#endif//#if( ISR_ENABLE_VPC3_INT_MAC_RESET == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_baudrate_detect                                        */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_BAUDRATE_DETECT == 1 )
void user_isr_baudrate_detect( void )
{
	//not used in our application
}//void user_isr_baudrate_detect( void )	
#endif//#if( ISR_ENABLE_VPC3_INT_BAUDRATE_DETECT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_gc_command                                         */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_GC_COMMAND == 1 )
void user_isr_new_gc_command( void )
{
	//not used in our application
}//void user_isr_new_gc_command( void )
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_GC_COMMAND == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_ssa_data                                           */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_SSA_DATA == 1 )
void user_isr_new_ssa_data(void)
{
	//not used in our application

VPC3_STRUC_SSA_BLOCK_PTR ssa_ptr;
	
	ssa_ptr = (VPC3_STRUC_SSA_BLOCK_PTR) VPC3_GET_SSA_BUF_PTR();
    //store the new address and the bit real_no_add_chg for the next startup
    /*
    print_string("\r\nNewAddr: ");
    print_hexbyte(ssa_ptr->address);
    print_hexbyte(ssa_ptr->no_add_chg);
    */
}
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_SSA_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_user_timer_clock (10ms)                                */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_USER_TIMER_CLOCK == 1 )
void user_isr_user_timer_clock( void )
{
	//simulation for counter module
	if( sSystem.output.counter[0] & 0x01 )
	{
		if( ++counter_timebase == sSystem.prm.time_base )
		{
			counter_timebase = 0x00;
			counter_value = (sSystem.prm.mode == 0) ? counter_value+1 : counter_value-1;
			sSystem.input.counter[0] = (UBYTE)(counter_value >> 8);
			sSystem.input.counter[1] = (UBYTE)(counter_value & 0x00FF);

			user.event |= VPC3_EV_NEW_INPUT_DATA;
		}//if( ++counter_timebase == sSystem.prm.time_base )	
	}//if( sSystem.output.counter[0] & 0x01 )	
}//void user_isr_user_timer_clock( void )
#endif//#if( ISR_ENABLE_VPC3_INT_USER_TIMER_CLOCK == 1 )

#ifdef DP_SUBSCRIBER
/*---------------------------------------------------------------------------*/
/* function: user_isr_dxb_out                                                */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_DXB_OUT == 1 )
void user_isr_dxb_out( void )
{
	user.event |= VPC3_EV_DXB_OUT;
}
#endif//#if( ISR_ENABLE_VPC3_INT_DXB_OUT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_dxb_link_error                                         */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_DXB_LINK_ERROR == 1 )
void user_isr_dxb_link_error(void)
{
	user.event |= VPC3_EV_DXB_LINK_ERROR;
}
#endif//#if( ISR_ENABLE_VPC3_INT_DXB_LINK_ERROR == 1 )

#endif//#ifdef DP_SUBSCRIBER


/*****************************************************************************/
/*  Copyright (C) profichip GmbH 2004. Confidential.                         */
/*****************************************************************************/
