/**************************  Filename: dp_user.c  ****************************/
/* ========================================================================= */
/*                                                                           */
/* 0000  000   000  00000 0  000  0   0 0 0000                               */
/* 0   0 0  0 0   0 0     0 0   0 0   0 0 0   0                              */
/* 0   0 0  0 0   0 0     0 0     0   0 0 0   0      Einsteinstrae 6        */
/* 0000  000  0   0 000   0 0     00000 0 0000       91074 Herzogenaurach    */
/* 0     00   0   0 0     0 0     0   0 0 0                                  */
/* 0     0 0  0   0 0     0 0   0 0   0 0 0          Tel: ++49-9132-744-200  */
/* 0     0  0  000  0     0  000  0   0 0 0    GmbH  Fax: ++49-9132-744-204  */
/*                                                                           */
/* ========================================================================= */
/*                                                                           */
/* Function:      Demo for PROFICHIP Extension Board AT89C5132               */
/*                This example simulates a profibus device                   */
/*                with 2 Byte Input and 2 Byte Output.                       */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Hardware requirements: ProfiChip Evaluation Board AT89C5132  (PA006101)   */
/*                        ProfiChip Evaluation Board VPC3+/C    (PA006103)   */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/* memory:  0000H ... 7FFFH: RAM                                             */
/*          8000H ... 8FFFH: MPI12x                                          */
/*          9000H ... 9FFFH: Reserved                                        */
/*          A000H ... AFFFH: FPGA                                            */
/*          B000H ... BFFFH: RTC                                             */
/*          C000H ... CFFFH: LCD                                             */
/*          D000H ... DFFFH: I/O Port 0                                      */
/*          E000H ... EFFFH: I/O Port 1                                      */
/*          F000H ... FFFFH: I/O Port 2                                      */
/*                                                                           */
/* ------------------------------------------------------------------------- */
/*                                                                           */
/* Technical support:       P. Fredehorst                                    */
/*                          Tel. : ++49-9132/744-214                         */
/*                          Fax. :              -204                         */
/*                          eMail: pfredehorst@profichip.com                 */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/* contents:

  - function prototypes
  - data structures
  - internal functions

*/
/*****************************************************************************/
/* include hierarchy */

#include "..\..\dp_inc\platform.h"
#include "..\..\dp_inc\dp_inc.h"

/*---------------------------------------------------------------------------*/
/* defines, structures                                                       */
/*---------------------------------------------------------------------------*/
// -- defines for user state
#define USER_STATE_CLEAR			((UBYTE)0x00)
#define USER_STATE_RUN				((UBYTE)0x01)
// -- defines for diagnostics
#define USER_TYPE_DPV0              ((UBYTE)0xFA)
#define USER_TYPE_PRM_OK            ((UBYTE)0xFB)
#define USER_TYPE_PRM_NOK           ((UBYTE)0xFC)
#define USER_TYPE_CFG_OK            ((UBYTE)0xFD)
#define USER_TYPE_CFG_NOK           ((UBYTE)0xFE)
#define USER_TYPE_APPL_RDY          ((UBYTE)0xFF)

typedef struct
{
    UBYTE 						state;
    UBYTE 						user_diag_active;
    UWORD 						old_diag;
    UBYTE 						user_diag[DIAG_BUFSIZE];
	UWORD						event;
	UBYTE 						application_ready;
	CFG_STRUCT 					real_cfg;
	UBYTE                       input[2];
	UBYTE                       output[2];
}USER_STRUC;

/*---------------------------------------------------------------------------*/
/* global user data definitions                                              */
/*---------------------------------------------------------------------------*/
VPC3_STRUC_ERRCB                vpc3_errcb; // error structure
USER_STRUC                      user;   	// user system structure

/*---------------------------------------------------------------------------*/
/* defines, structures and variables for our demo application                */
/*---------------------------------------------------------------------------*/
//default configuration data for startup
#define UserCfgDataLength ((UBYTE)0x01)
ROMCONST__ UBYTE DefCfg[1] = { 0x31 };

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
void read_input_data( void )
{
VPC3_UNSIGNED8_PTR  input_buf_ptr; 	// pointer to input buffer

    // write DIN data to VPC3
    if(input_buf_ptr = vpc3_get_dinbufptr ())            // get pointer to DIN data
    {
        //todo:
        //handle here data (slave --> master) 
		memcpy( input_buf_ptr, &user.input[0], dp_sys.inp_data_len );

        // the user makes a new Din-Buffer available in the state N
        dp_sys.vpc3_con = VPC3_INPUT_UPDATE();
    }//if(input_buf_ptr = vpc3_get_dinbufptr ())
}//void read_input_data( void )	

/*---------------------------------------------------------------------------*/
/* function: user_alarm ( is also called from alarm state machine !!!! )     */
/*---------------------------------------------------------------------------*/
UBYTE user_alarm( UBYTE alarm_type, UBYTE check_diag_flag )
{
VPC3_UNION_DIAG_PTR tmp_diag;
UBYTE 				ret_value;
UBYTE               ext_diag;
UBYTE               len_diag;
UBYTE 				error;
UWORD 				diag;

    ret_value = 0x00;

    //don't send diagnostic twice!
    if(	   ( diag  != user.old_diag         )
    	&& ( FALSE == user.user_diag_active )
      )	
    {
        memset( &user.user_diag[0], 0x00, sizeof( user.user_diag ) );
        tmp_diag.byte_ptr = user.user_diag;

        ext_diag = 0x00;
        len_diag = 0x00;
        
        switch( alarm_type )
        {
            case USER_TYPE_CFG_OK:
            {
                ext_diag = STAT_DIAG_SET;
                break;
            }//case USER_TYPE_CFG_OK:

        	case USER_TYPE_APPL_RDY:
            case USER_TYPE_PRM_NOK:
            case USER_TYPE_PRM_OK:
            default:
            {
                ext_diag = 0x00;
                len_diag = 0x00;
                break;
            }//default:
        }//switch(user[vpc3_channel].diag_byte)

        user.user_diag_active = TRUE;
            
        error = set_diagnosis( tmp_diag, len_diag, ext_diag, check_diag_flag  );

        if( error == DP_OK )
        {
            user.old_diag = diag;
            if( alarm_type >= USER_TYPE_PRM_OK )
            {
                user.user_diag_active = FALSE;
            }//if( alarm_type >= USER_TYPE_PRM_OK )
            ret_value = DP_OK;
        }//if( error == DP_OK )
        else
        {
        	user.user_diag_active = FALSE;
			
			#ifdef RS232_SERIO
            	print_string("DIAG_DP_Error: ");
            	print_hexbyte(error);
            	print_hexbyte(alarm_type);
			#endif//#ifdef RS232_SERIO

            ret_value = error;
        }//else of if( error == DP_OK )
    }//if(	   ( diag  != user.old_diag         ) ...

    return ret_value;
}//UBYTE user_alarm( UBYTE alarm_type, UBYTE check_diag_flag )

/*---------------------------------------------------------------------------*/
/* function: user_alarm ( is also called from alarm state machine !!!! )     */
/*---------------------------------------------------------------------------*/
void application_ready( void )
{
	//todo
	//make here your own initialization	
	user.input[0] = *READ_PORT1;
	user.input[1] = *READ_PORT2;
		
	//read input data
    read_input_data();
    //reset Diag.Stat 
    user_alarm( USER_TYPE_APPL_RDY, FALSE );
    user.application_ready = TRUE;
}//void application_ready( void )	

/*---------------------------------------------------------------------------*/
/* user main function                                                        */
/*---------------------------------------------------------------------------*/
void user_main(void)
{
VPC3_UNSIGNED8_PTR  		output_buf_ptr; // pointer to output buffer
DP_ERROR_CODE				error;
UBYTE           			outp_state; 	// state of output data

    /*-----------------------------------------------------------------------*/
    /* init user data                                                        */
    /*-----------------------------------------------------------------------*/
	memset( &user, 0, sizeof(user) );

    /* TRUE  deactivates diagnosis handling ! */
    /* FALSE activates   diagnosis handling ! */
    user.user_diag_active  = FALSE;
	user.application_ready = FALSE;
    user.state = USER_STATE_CLEAR;

    /*-----------------------------------------------------------------------*/
    /* initialize VPC3                                                       */
    /*-----------------------------------------------------------------------*/

    //todo: 
    //insert your real configuration data here
    user.real_cfg.length = UserCfgDataLength; // length of configuration data
	memcpy( &user.real_cfg.cfg_data[0], &DefCfg[0], user.real_cfg.length );

	#ifdef EvaBoard_AT89C5132
    	error = vpc3_initialization( (*READ_PORT0 & 0x7F), user.real_cfg ); 	// address of slave; PORT0
	#else
    	error = vpc3_initialization( DP_ADDR, user.real_cfg );					// address of slave
	#endif//#ifdef EvaBoard_AT89C5132
    
    if( DP_OK == error )
    {
    	// set microcontroller interrupt to falling edge
    	SET_FALLING_EDGE__;
    	// enable microcontroller interrupt
    	ENABLE_VPC3_INT__;

        //todo: before startup the VPC3+, make here your own initialzations 
        
        START_VPC3();
    }
    else
    {
        vpc3_errcb.error_code = error;
        fatal_error( _DP_USER, __LINE__, &vpc3_errcb );
    }//if( DP_OK == error )

    /*-----------------------------------------------------------------------*/
    /* endless demo operation loop                                           */
    /*-----------------------------------------------------------------------*/
    while( 1 )
    {
        /*-------------------------------------------------------------------*/
        /* trigger watchdogs                                                 */
        /*-------------------------------------------------------------------*/
        // toggle user watchdog
        VPC3_RESET_USER_WD();   // toggle user watchdog

		#ifdef RS232_SERIO
        	if(rec_counter > 0)
        	{
				DISABLE_ALL_INTERRUPT_BIT__;
				
            	print_serial_inputs();
            	
            	ENABLE_ALL_INTERRUPT_BIT__;
        	}
		#endif//#ifdef RS232_SERIO


        /*-------------------------------------------------------------------*/
        /* internal state machine                                            */
        /*-------------------------------------------------------------------*/
        if( user.state == USER_STATE_CLEAR )
        {
            // clear data
			memset( &user.input[0] , 0, 2 );
			memset( &user.output[0], 0, 2 );

			#ifdef EvaBoard_AT89C5132
            	*WRITE_PORT1 = 0x00;
            	*WRITE_PORT2 = 0x00;
			#endif//#ifdef EvaBoard_AT89C5132

            user.state = USER_STATE_RUN;
        }//if( user.state == USER_STATE_CLEAR )

        /*-------------------------------------------------------------------*/
        /* dpv1 statemachines                                                */
        /*-------------------------------------------------------------------*/
        vpc3_dpv1_statemachine();

        /*-------------------------------------------------------------------*/
        /* VPC3+ DP-state                                                    */
        /*-------------------------------------------------------------------*/
        switch(VPC3_GET_DP_STATE())
        {
            case WAIT_PRM:
            case WAIT_CFG:
            case DP_ERROR:
            {
                // set LED's
                CLR_LED_YLW__;
                SET_LED_RED__;
                
                if( user.event & VPC3_EV_DX_OUT )
				{
					user.event &= ~VPC3_EV_DX_OUT;       // clear event
				}		
                break;
            }

            case DATA_EX:
            {
                // set LED's
                SET_LED_YLW__;
                CLR_LED_RED__;
				
				if(    ( user.application_ready == TRUE )
					&& ( user.state == USER_STATE_RUN   )
				  )
				{
					/*-------------------------------------------------------------------*/
        			/* profibus output                                                   */
        			/*-------------------------------------------------------------------*/
					if( user.event & VPC3_EV_DX_OUT )
					{
						user.event &= ~VPC3_EV_DX_OUT;       // clear event
        			    if( output_buf_ptr = vpc3_get_doutbufptr (&outp_state) )
        			    {
        			    	//todo:
        			    	//handle here data (master --> slave) 
                			memcpy( &user.output[0], output_buf_ptr, dp_sys.outp_data_len);
                			
        					/*-----------------------------------------------------------*/
        					/* user application                                          */
        					/*-----------------------------------------------------------*/
							#ifdef EvaBoard_AT89C5132
            					*WRITE_PORT1 = user.output[0];
            					*WRITE_PORT2 = user.output[1];
							#endif//#ifdef EvaBoard_AT89C5132
                		}//if( user.output_buf_ptr = vpc3_get_doutbufptr (&outp_state) ) 	
        			}//if( user.event & VPC3_EV_DX_OUT )
        			
        			/*-------------------------------------------------------------------*/
        			/* user application                                                  */
        			/*-------------------------------------------------------------------*/
					#ifdef EvaBoard_AT89C5132
						if( user.input[0] != *READ_PORT1 )
						{
							user.input[0] = *READ_PORT1;
							user.event |= VPC3_EV_NEW_INPUT_DATA;
						}
						
						if( user.input[1] != *READ_PORT2 )
						{
							user.input[1] = *READ_PORT2;
							user.event |= VPC3_EV_NEW_INPUT_DATA;
						}
					#endif//#ifdef EvaBoard_AT89C5132

        			/*-------------------------------------------------------------------*/
        			/* profibus input                                                    */
        			/*-------------------------------------------------------------------*/
					if( user.event & VPC3_EV_NEW_INPUT_DATA )
					{
						read_input_data();
						user.event &= ~VPC3_EV_NEW_INPUT_DATA;
					}
				}//if(    ( user.application_ready == TRUE ) ...
				
                break;
            }//case DATA_EX:

            default:
            {
                vpc3_errcb.error_code = VPC3_GET_DP_STATE();
                fatal_error( _DP_USER, __LINE__, &vpc3_errcb );
                break;
            }
        }//switch(VPC3_GET_DP_STATE())

        /*-------------------------------------------------------------------*/
        /* handle here profibus interrupt events                             */
        /*-------------------------------------------------------------------*/
    	if( user.event & VPC3_EV_NEW_CFG_DATA )
    	{
        	application_ready();
        	user.event &= ~VPC3_EV_NEW_CFG_DATA;    // clear event
    	}

    }//while(1)
}//void user_main(void)



/*---------------------------------------------------------------------------*/
/* delay                                                                     */
/*---------------------------------------------------------------------------*/
void delay( void )
{
UBYTE i,j;

    for(i = 0; i < 255; i++)
    {
        for(j = 0; j < 255; j++);
    }
}

/*---------------------------------------------------------------------------*/
/*                                                                           */
/*   function:     fatal_error                                               */
/*                                                                           */
/*---------------------------------------------------------------------------*/
void fatal_error( DP_ERROR_FILE file, UWORD line, VPC3_ERRCB_PTR errcb_ptr)
{
	#ifdef RS232_SERIO
    	do
    	{
    	    // wait!
    	}
    	while( snd_counter > 80);
    	
    	print_string("\r\nFATAL_ERROR:");
    	print_string("\r\nFile: ");
    	print_hexbyte(file);
    	print_string("\r\nLine: ");
    	print_hexword(line);
    	print_string("\r\nFunction: ");
    	print_hexbyte(errcb_ptr->function);
    	print_string("\r\nError_Code: ");
    	print_hexbyte(errcb_ptr->error_code);
    	print_string("\r\nDetail: ");
    	print_hexbyte(errcb_ptr->detail);
    	print_string("\r\ncn_id: ");
    	print_hexbyte(errcb_ptr->cn_id);
	#endif//#ifdef RS232_SERIO

	#ifdef EvaBoard_AT89C5132
    	*WRITE_PORT0 = errcb_ptr->error_code;
    	*WRITE_PORT1 = file;
    	*WRITE_PORT2 = (UBYTE)line;
	#endif//#ifdef EvaBoard_AT89C5132

    SET_LED_YLW__;
    SET_LED_RED__;

    while(1)
    {
		#ifdef RS232_SERIO
        	if(rec_counter > 0)
        	{
            	print_serial_inputs();
        	}
		#endif//#ifdef RS232_SERIO

        TOGGLE_LED_RED__;
        TOGGLE_LED_YLW__;

        delay();
    }//while(1)
}//void fatal_error( DP_ERROR_FILE file, UWORD line, VPC3_ERRCB_PTR errcb_ptr)


/*---------------------------------------------------------------------------*/
/* function: user_isr_new_prm_data                                           */
/*                                                                           */
/* todo  : check the prm-data                                                */
/* return: DP_OK  - prm-data OK                                              */
/*         DP_NOK - prm-data not OK                                          */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_PRM_DATA == 1 )
UBYTE user_chk_new_prm_data( VPC3_UNSIGNED8_PTR prm_ptr, UBYTE len )
{
VPC3_STRUC_PRM_PTR 			prm_data;
UBYTE              			ret_value;

    ret_value = DP_OK;

    if( len == PRM_LEN_DPV1 )
    {
        prm_data = (VPC3_STRUC_PRM_PTR)prm_ptr;

        //check DPV1 Status
        if(    ( prm_data->dpv1_status_1 != 0x00 )
        	|| ( prm_data->dpv1_status_2 != 0x00 )
            || ( prm_data->dpv1_status_3 != 0x00 )
          )
        {
            ret_value = DP_PRM_DPV1_STATUS;
        }//if(    (( prm_data->dpv1_status_2 & 0x02 ) != 0x00 ) ...
    }//if( len >= PRM_LEN_DPV1 )
    else
    {
        ret_value = DP_PRM_LEN_ERROR;
    }//else of if( len >= PRM_LEN_DPV1 )

    if( ( VPC3_GET_DP_STATE() == DATA_EX ) && ( ret_value == DP_OK ) )
    {
        //don't send diagnostic here
    }//if( ( VPC3_GET_DP_STATE() == DATA_EX ) && ( ret_value == DP_OK ) )
    else
    {
        user_alarm( USER_TYPE_PRM_OK, FALSE );
    }//else of if( ( VPC3_GET_DP_STATE() == DATA_EX ) && ( ret_value == DP_OK ) )

	return ret_value;	
}//UBYTE user_chk_new_prm_data(VPC3_UNSIGNED8_PTR prm_ptr, UBYTE len)
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_PRM_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_cfg_data 											 */
/*									                                         */
/* todo  : check cfg data   		                                         */
/*									                                         */
/* return:	VPC3_CFG_OK      0												 */		
/*			VPC3_CFG_FAULT   1												 */	
/*			VPC3_CFG_UPDATE  2								                 */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_CFG_DATA == 1 )
UBYTE user_chk_new_cfg_data(void)
{
VPC3_UNSIGNED8_PTR  readcfg_ptr;   	// pointer read config buffer
VPC3_UNSIGNED8_PTR  cfg_ptr;   		// pointer check config buffer
UBYTE               cfg_len;
UBYTE               ret_value;

    ret_value = VPC3_CFG_OK;
    cfg_len = VPC3_GET_CFG_LEN();

    if( cfg_len == VPC3_GET_READ_CFG_LEN() )
    {
    	readcfg_ptr = VPC3_GET_CFG_BUF_PTR();
    	cfg_ptr  	= VPC3_GET_CFG_BUF_PTR();

        if( memcmp( readcfg_ptr, cfg_ptr, cfg_len ) != 0 )
        {
            ret_value = VPC3_CFG_FAULT;
        }
    }//if( cfg_len == VPC3_GET_READ_CFG_LEN() )
    else
    {
        ret_value = VPC3_CFG_FAULT;
    }//if( cfg_len != real_cfg_len )

    if( ret_value == VPC3_CFG_OK )
    {
        user_alarm( USER_TYPE_CFG_OK, FALSE );
        user.event |= VPC3_EV_NEW_CFG_DATA;
    }//if( ret_value == VPC3_CFG_UPDATE )
	return ret_value;		
}//UBYTE user_chk_new_cfg_data(void)
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_CFG_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_go_leave_data_ex                                       */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_GO_LEAVE_DATA_EX == 1 )
void user_isr_go_leave_data_ex( void )
{
	if( VPC3_GET_DP_STATE() != DATA_EX )
   	{
        user.state = USER_STATE_CLEAR;
   		user.application_ready = FALSE;
    }//if( VPC3_GET_DP_STATE() != DATA_EX )		
}//void user_isr_go_leave_data_ex( void )
#endif//#if( ISR_ENABLE_VPC3_INT_GO_LEAVE_DATA_EX == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_dx_out                                                 */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_DX_OUT == 1 )
void user_isr_dx_out( void )
{
	user.event |= VPC3_EV_DX_OUT;
}	
#endif//#if( ISR_ENABLE_VPC3_INT_DX_OUT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_diag_buf_changed                                       */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_DIAG_BUF_CHANGED == 1 )
void user_isr_diag_buf_changed( void )
{
    // diagnosis buffer has been changed
    user.user_diag_active = FALSE;
    // Fetch new diagnosis buffer
    dp_sys.diag_buf_ptr.byte_ptr = vpc3_get_diagbufptr();
}//void user_isr_diag_buf_changed( void )
#endif//#if( ISR_ENABLE_VPC3_INT_DIAG_BUF_CHANGED == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_wd_timeout                                         */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_WD_DP_TIMEOUT == 1 )
void user_isr_new_wd_dp_timeout( void )
{
	reset_diagnostic_buffer();
}//void user_isr_new_wd_dp_timeout( void )
#endif//#if( ISR_ENABLE_VPC3_INT_WD_DP_TIMEOUT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_ext_prm_data                                       */
/*                                                                           */
/* todo  : check the prm-data                                                */
/* return: DP_OK  - prm-data OK                                              */
/*         DP_NOK - prm-data not OK                                          */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_EXT_PRM_DATA == 1 )
UBYTE user_chk_new_ext_prm_data( VPC3_STRUC_PRM_BLOCK_PTR prmblock_ptr, UBYTE len )
{
	//not used
	return DP_OK;
}
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_EXT_PRM_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_mac_reset                                              */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_MAC_RESET == 1 )
void user_isr_mac_reset( void )
{
	//not used in our application
}	
#endif//#if( ISR_ENABLE_VPC3_INT_MAC_RESET == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_baudrate_detect                                        */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_BAUDRATE_DETECT == 1 )
void user_isr_baudrate_detect( void )
{
	//not used in our application
}//void user_isr_baudrate_detect( void )	
#endif//#if( ISR_ENABLE_VPC3_INT_BAUDRATE_DETECT == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_gc_command                                         */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_GC_COMMAND == 1 )
void user_isr_new_gc_command( void )
{
	//not used in our application
}//void user_isr_new_gc_command( void )
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_GC_COMMAND == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_new_ssa_data                                           */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_NEW_SSA_DATA == 1 )
void user_isr_new_ssa_data(void)
{
	//not used in our application
}
#endif//#if( ISR_ENABLE_VPC3_INT_NEW_SSA_DATA == 1 )

/*---------------------------------------------------------------------------*/
/* function: user_isr_user_timer_clock (10ms)                                */
/*---------------------------------------------------------------------------*/
#if( ISR_ENABLE_VPC3_INT_USER_TIMER_CLOCK == 1 )
void user_isr_user_timer_clock( void )
{
	//not used
}//void user_isr_user_timer_clock( void )
#endif//#if( ISR_ENABLE_VPC3_INT_USER_TIMER_CLOCK == 1 )


/*****************************************************************************/
/*  Copyright (C) profichip GmbH 2005. Confidential.                         */
/*****************************************************************************/
